<?php

namespace EventPortal\Domain\Agenda;

use Carbon\Carbon;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Component\Option;
use Illuminate\Support\Arr;

class SessionInput
{
    /** @var array */
    private $input;

    /** @var \Carbon\Carbon */
    private $date;

    public function __construct(array $input)
    {
        $this->input = $input;
        $this->date = now();
    }

    public static function from(array $input): self
    {
        return new self($input);
    }

    public function forceDate(Carbon $date): self
    {
        $this->date = $date;

        return $this;
    }

    public function formatForDatabase(): array
    {
        return collect($this->input)
            ->mapWithKeys(function ($value, $key) {
                return [SessionKeyMap::getDatabaseKeyFromBrowser($key) => $value];
            })
            ->map(function ($value, $key) {
                if ($this->notATimeField($key)) {
                    return $value;
                }

                return $this->timeAsHourMinute($value);
            })
            ->map(function ($value, $key) {
                if ($key === 'event_venue_id') {
                    return Arr::get($value, 'value');
                }

                return $value;
            })
            ->toArray();
    }

    public function formatForBrowser(?Session $session = null): array
    {
        return collect($this->input)
            ->mapWithKeys(function ($value, $key) {
                return [SessionKeyMap::getBrowserKeyFromDatabase($key) => $value];
            })
            ->map(function ($value, $key) {
                if (self::notATimeField($key)) {
                    return $value;
                }

                return ['startTime' => Carbon::parse($value)->format('h:i A')];
            })
            ->put('session-speakers', $this->getEventSpeakers($session))
            ->put('session-venue', $this->getVenue($session))
            ->toArray();
    }

    public function getInput(): array
    {
        return $this->input;
    }

    private function notATimeField(string $key): bool
    {
        $timeFields = [
            'starting_at',
            'ending_at',
            'session-start-time',
            'session-end-time',
        ];

        return in_array($key, $timeFields, true) === false;
    }

    /**
     * @param array|string $value
     */
    private function timeAsHourMinute($value): string
    {
        $time = is_array($value) ? $value['startTime'] : $value;
        $time = Carbon::parse($time);

        return Carbon::create(
            $this->date->year,
            $this->date->month,
            $this->date->day,
            $time->hour,
            $time->minute
        );
    }

    private function getEventSpeakers(?Session $session): array
    {
        if ($session === null || $session->eventSpeakers() === null) {
            return [];
        }

        return $session->eventSpeakers->map->asOption()->toArray();
    }

    private function getVenue(?Session $session): Option
    {
        if ($session === null || $session->eventVenue === null) {
            return Option::make('');
        }

        return $session->eventVenue->asOption();
    }
}
