<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\ActionList\Models\ActionList;
use EventPortal\Domain\ActionList\Models\Task;
use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class ActionListTest extends TestCase
{
    use DatabaseMigrations;

    /** @test */
    public function can_get_child_tasks(): void
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id,
        ]);

        $list->tasks()->saveMany(factory(Task::class, 5)->make());

        $this->assertCount(5, $list->tasks);
    }

    /** @test */
    public function can_complete_all_tasks(): void
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id,
        ]);

        $list->tasks()->saveMany(factory(Task::class, 5)->make());

        $this->assertCount(5, $list->tasks);
        $this->assertCount(0, $list->tasks()->completed()->get());
        $this->assertCount(5, $list->tasks()->incompleted()->get());

        $list->complete();

        $this->assertCount(5, $list->tasks()->completed()->get());
        $this->assertCount(0, $list->tasks()->incompleted()->get());
    }

    /** @test */
    public function can_pass_user_to_complete(): void
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id,
        ]);

        $list->tasks()->saveMany(factory(Task::class, 5)->make());
        $myAdmin = factory(Admin::class)->create();
        $otherAdmin = factory(Admin::class)->create();

        $this->actingAsAdmin($myAdmin);

        $list->complete($otherAdmin->user);

        $this->assertSameModel($otherAdmin->user, $list->tasks->first()->completedBy);
    }

    /** @test */
    public function can_pass_no_user_to_complete_and_see_my_user_as_completed_by(): void
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id,
        ]);

        $list->tasks()->saveMany(factory(Task::class, 5)->make());
        $myAdmin = factory(Admin::class)->create();

        $this->actingAsAdmin($myAdmin);

        $list->complete();

        $this->assertSameModel($myAdmin->user, $list->tasks->first()->completedBy);
    }
    
    /** @test */
    public function can_get_outstanding_tasks_count(): void
    {
        $event = factory(Event::class)->create();
        $list = factory(ActionList::class)->create([
            'action_listable_type' => Event::class,
            'action_listable_id' => $event->id,
        ]);

        $list->tasks()->saveMany(factory(Task::class, 5)->make());

        $this->assertEquals(5, $list->numOutstandingTasks);

        $list->complete();

        $this->assertEquals(0, $list->numOutstandingTasks);
    }
}
